public class BookShelf
{
    // This is a class to model a shelf of books using an array.

    private Book[] bookArray;
    private int numBooks;

    public BookShelf()
    {
        bookArray = new Book[100];
        numBooks = 0;
    }

    public void insertAtEnd(Book newBook)
    {
        // Inserts book into the next available position

        if (numBooks == bookArray.length) {
            System.out.println("BOOKSHELF FULL: DID NOT INSERT " + newBook);
            return;	// exit method immediately
        }
        bookArray[numBooks] = newBook;
        numBooks++;
    }

    public String toString()
    {
        String result;
        result = "Number of books on shelf: " + numBooks + "\n";

        // The following loop goes through the array of books and
        // calls toString on each book on the shelf, adding this
        // string to the result string that will be returned
        // from this method.

        for (int i = 0; i < numBooks; i++)
            result += "bookArray[" + i + "]: " + bookArray[i].toString() + "\n";

        return result;
    }        

    // CLASS EXERCISES START BELOW:

    public double getTotalCost()
    {
	// Method to return the total cost of all books on the bookshelf
        double total = 0.0;

        for (int index = 0; index < numBooks; index++) {
        	total += bookArray[index].getPrice();
        }

        return total;   
    }

    public void insert1(Book newBook)
    {
	// Method to insert a new book into the first position of the
	// bookshelf (index 0 of the array) and shift all subsequent
	// books over one position

    	 if (numBooks == bookArray.length) {
             System.out.println("BOOKSHELF FULL: DID NOT INSERT " + newBook);
             return;	// exit method immediately
         }    	

    	 for (int index=numBooks-1; index >= 0; index--)
    	 {	 
    		 bookArray[index+1]= bookArray[index];
    	 }
    	 bookArray[0] = newBook;
    	 numBooks++;

    }

    public void insert2(Book newBook)
    {
	// Method to insert the new book into the bookshelf so that
	// books are always in non-decreasing order by number of pages.
	// Precondition: Bookshelf already is sorted in non-decreasing
	// order by number of pages.
    	
      	 if (numBooks == bookArray.length) {
             System.out.println("BOOKSHELF FULL: DID NOT INSERT " + newBook);
             return;	// exit method immediately
         }    	
      	 
      	 int position=0;
      	 while(position < numBooks && bookArray[position].getPages()  < newBook.getPages()) 
         {
      		 position++;
      	 }
      	 
      	 // BONUS #1: Alternate way to search for the first book that has at least as
         // many pages as the new book we want to insert
 /*     	 
      	 for(position = 0; position <= numBooks-1;
      	      position++)
      	 {
      		 if (bookArray[position].getPages()
      				 >= newBook.getPages())
      			 break;
      	 }
 */     	
 
     	 // BONUS #2: Ordering based on author instead of pages using compareTo
/*      	 
      	 int position=0;
      	 for (position = 0; position <= numBooks-1;
      	 			position++) {
      		String authorInArray = bookArray[position].getAuthor();
      		if (authorInArray.compareTo(newBook.getAuthor()) >= 0 )
      			break;
      	 }
*/      	 	

         // Once the insert position is found, shift all subsequent books and insert new book in 
      	 
      	 for (int i=numBooks-1; i >= position; i--)
      	 {
      		 bookArray[i+1]= bookArray[i];
      	 }
      	 bookArray[position] = newBook;
      	 numBooks++;
      	 
    }

    public void remove1()
    {
	 // Method to remove the first book on the bookshelf (if any)
	 // any shift subsequent books over one position to fill in
	 // the gap left by the removed book

         if (numBooks == 0)
                 return;

         for (int i = 1; i <= numBooks-1; i++)
      	 {
      		 bookArray[i-1]= bookArray[i];
         }
         bookArray[numBooks-1] = null;
         numBooks--;
    }

    public void remove2(String author)
    {
	// Method to remove the first book on the book shelf that 
	// has an author that matches the author given in the parameter.
	// Subsequent books are shifted one position over to fill in
	// the gap, if a matching book is found.

    	if (numBooks == 0)
    		return;
    	
    	int position = 0;
    	while (position < numBooks &&
    		bookArray[position].getAuthor()
    		           .equals(author)==false)	
    	{
    		position++;
    	}
    	
    	if (position < numBooks) // find a match?
    	{
    		for (int i=position+1; i<=numBooks-1; i++)
    			bookArray[i-1] = bookArray[i];
    		bookArray[numBooks-1] = null;
    		numBooks--;
    	}

    }

    public void remove3()
    {
	// Method to remove the book with the maximum number of pages
	// from the bookshelf, shifting all subsequent books over one
	// position to fill in the gap.

    	if (numBooks == 0)
    		return;
    	
    	// We have at least one book!
    	
    	int maxBook = 0;	// index of max book
    	for (int i = 1; i < numBooks; i++) {
    		if (bookArray[i].getPages() 
    				> bookArray[maxBook].getPages())
    			maxBook = i;
    	}

    	for (int i = maxBook+1; i <= numBooks-1; i++) {
    		bookArray[i-1] = bookArray[i];
    	}
    	bookArray[numBooks-1] = null;
    	numBooks--;
    }


    public BookShelf filter1(int pages)
    {
        // Creates a new bookshelf containing references
        // to books in the current (i.e. this) bookshelf
        // that have at least the number of pages given
        // in the parameter

         BookShelf newShelf = new BookShelf();

         for (int i=0; i < numBooks; i++) {
        	 if (this.bookArray[i].getPages() >= pages)
        		 newShelf.insertAtEnd(bookArray[i]);
         }

         return newShelf;
    }


}
